"""
Anyparser LlamaIndex Integration
"""

__version__ = "0.0.2"

from typing import List, Literal, Optional, Union

from anyparser_core import (
    Anyparser,
    AnyparserCrawlResult,
    AnyparserPdfResult,
    OcrLanguage,
    OCRPreset,
)
from anyparser_core.options import AnyparserOption
from llama_index.core import Document


class AnyparserReader:
    """Reader for Anyparser documents."""

    def __init__(
        self,
        api_key: Optional[str] = None,
        api_url: Optional[str] = None,
        format: Literal["json", "markdown", "html"] = "json",
        model: Literal["text", "ocr", "vlm", "lam", "crawler"] = "text",
        encoding: Literal["utf-8", "latin1"] = "utf-8",
        image: Optional[bool] = None,
        table: Optional[bool] = None,
        ocr_language: Optional[List[OcrLanguage]] = None,
        ocr_preset: Optional[OCRPreset] = None,
        max_depth: Optional[int] = None,
        max_executions: Optional[int] = None,
        strategy: Optional[Literal["LIFO", "FIFO"]] = None,
        traversal_scope: Optional[Literal["subtree", "domain"]] = None,
    ):
        """Initialize with parameters.

        Args:
            api_key: Anyparser API key
            api_url: Anyparser API URL
            format: Output format (json, markdown, html)
            model: Processing model (text, ocr, vlm, lam, crawler)
            encoding: Text encoding (utf-8, latin1)
            image: Whether to extract images
            table: Whether to extract tables
            ocr_language: List of languages for OCR
            ocr_preset: OCR preset configuration
            max_depth: Maximum crawl depth for web crawling
            max_executions: Maximum number of pages to crawl
            strategy: Crawling strategy (LIFO, FIFO)
            traversal_scope: Crawling scope (subtree, domain)
        """
        self.parser = Anyparser(
            options=AnyparserOption(
                api_key=api_key,
                api_url=api_url,
                format=format,
                model=model,
                encoding=encoding,
                image=image,
                table=table,
                ocr_language=ocr_language,
                ocr_preset=ocr_preset,
                max_depth=max_depth,
                max_executions=max_executions,
                strategy=strategy,
                traversal_scope=traversal_scope,
            )
        )

    async def _parse_document(self, file_path: str) -> List[Document]:
        """Parse a single document and convert to LlamaIndex Documents."""
        result = await self.parser.parse(file_path)

        if isinstance(result, str):
            # Handle markdown/html string format
            return [Document(text=result)]

        documents = []
        for item in result:
            if isinstance(item, AnyparserPdfResult):
                # Handle PDF results
                for page in item.items:
                    doc = Document(
                        text=page.markdown or page.text,
                        extra_info={
                            "page_number": page.page_number,
                            "file_name": item.original_filename,
                            "rid": item.rid,
                            "total_characters": item.total_characters,
                            "checksum": item.checksum,
                        },
                    )
                    documents.append(doc)
            elif isinstance(item, AnyparserCrawlResult):
                # Handle web crawl results
                for url_item in item.items:
                    doc = Document(
                        text=url_item.markdown,
                        extra_info={
                            "url": url_item.url,
                            "title": url_item.title,
                            "rid": item.rid,
                            "status_code": url_item.status_code,
                            "status_message": url_item.status_message,
                            "crawled_at": url_item.crawled_at,
                            "total_characters": url_item.total_characters,
                        },
                    )
                    documents.append(doc)
            else:
                # Handle base result type
                doc = Document(
                    text=item.markdown or "No content",
                    extra_info={
                        "file_name": item.original_filename,
                        "rid": item.rid,
                        "total_characters": item.total_characters,
                        "checksum": item.checksum,
                    },
                )
                documents.append(doc)

        return documents

    async def load_data(self, file_path: Union[str, List[str]]) -> List[Document]:
        """Load data from the file(s) and convert to LlamaIndex Documents.

        Args:
            file_path: Path to file(s) or URL(s) to process. Can be:
                - Single file path (e.g., "document.pdf")
                - Multiple file paths (e.g., ["doc1.pdf", "doc2.docx"])
                - Web URL (e.g., "https://example.com")
                - Image file for OCR (e.g., "scan.png")

        Returns:
            List of LlamaIndex Document objects containing the parsed content
        """
        import asyncio

        if isinstance(file_path, str):
            file_paths = [file_path]
        else:
            file_paths = file_path

        tasks = [self._parse_document(fp) for fp in file_paths]
        results = await asyncio.gather(*tasks)
        return [doc for result in results for doc in result]
